% stochastic model of the AND gate
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions
% observ: observed chemical species
% plot_on: turns the plotting on
%
% Outputs:
% X_mean: time course of observed chemical species
%
% sample calls
%X_init = [0,0,0,1,0,0,0]'; %a
%X_init = [0,1200,0,1,0,0,0]'; %b
%X_init = [1200,0,0,1,0,0,0]'; %c
%X_init = [1200,1200,0,1,0,0,0]'; %d
% and_ssa(10000,10,X_init,[7],1);
function [X_mean] = and_ssa(t_end,t_step,X_init,observ,plot_on)

% X = [x, y, x*, DNA, DNA*, mRNA, z]

runs = 1;

disp('SSA - AND gate');

% -> define the rate constant vector c
A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

k_association_ind    = 0.00012;
k_dissociation_ind   = 0.4791;


k1 = k_association_ind;     % binding of x with y to form the complex x*
k2 = k_dissociation_ind;    % dissociation of complex x*

k3 = k_association_OR3;
k4 = k_dissociation_OR2;

k5 = k_transcription;   % transcription of protein z mRNA
k6 = k_translation;     % translation of protein z
k7 = k_degrad_mRNA;     % degradation of protein z mRNA
k8 = k_degrad_mono;     % degradation of protein z

tmp = conc_num_to_mol(X_init(2));
gamma_ind = 1;
Kd1 = (k_dissociation_ind/k_association_ind)^(1/gamma_ind);


delez = tmp^gamma_ind/(tmp^gamma_ind + Kd1^gamma_ind);

c1 = k1 * 2 * ((10^9)/AV)^2;    
c2 = k2;                        

c3 = k3 * 2 * ((10^9)/AV)^2;    
c4 = k4;                       

c5 = k5;                 
c6 = k6;                 
c7 = k7;                 
c8 = k8;                 

c = [c1, c2, c3, c4, c5, c6, c7, c8];

% -> define the reactions (i.e. the reactants and the products matrix)
% X = [x, y, x*, DNA, DNA*, mRNA, z]

r1 = [1, 1, 0, 0, 0, 0, 0]; p1 = [0, 0, 1, 0, 0, 0, 0]; % x + y  -> x*
r2 = [0, 0, 1, 0, 0, 0, 0]; p2 = [1, 1, 0, 0, 0, 0, 0]; % x* -> x + y 
r3 = [0, 0, 2, 1, 0, 0, 0]; p3 = [0, 0, 0, 0, 1, 0, 0]; % 2(x_y2) + DNA -> DNA*        
r4 = [0, 0, 0, 0, 1, 0, 0]; p4 = [0, 0, 2, 1, 0, 0, 0]; % 2(x_y2) + DNA -> DNA*        

r5 = [0, 0, 0, 0, 1, 0, 0]; p5 = [0, 0, 0, 0, 1, 1, 0]; % DNA* -> mRNA + DNA*
r6 = [0, 0, 0, 0, 0, 1, 0]; p6 = [0, 0, 0, 0, 0, 1, 1]; % mRNA -> mRNA + z
r7 = [0, 0, 0, 0, 0, 1, 0]; p7 = [0, 0, 0, 0, 0, 0, 0]; % mRNA -> 0
r8 = [0, 0, 0, 0, 0, 0, 1]; p8 = [0, 0, 0, 0, 0, 0, 0]; % z -> 0

reactants = [r1;r2;r3;r4;r5;r6;r7;r8];
products = [p1;p2;p3;p4;p5;p6;p7;p8];


% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps
t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

for k=1:runs
    [X] = ssa(c,reactants,products,X_init,t_end,t_step);
    
    for l=1:length(observ)
        X_total(k,l,:)=X(observ(l),:);
    end;
end;

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)

    for l=1:length(observ)
        figure(l), plot(t, X_means(l,:));
        axis([0,t_end,0,1400]);
        xlabel('Time [s]');
        ylabel('Number of molecules of protein z');
        box off;
    end;
end