% deterministic model of the biological driver
%
% Inputs:
% x: input signal
% m_0, y_0: initial state of the systems
% dt: time step size
% t_end: simulation time
% plot_on: turns the plotting on
% 
% Outputs:
% y: time evolution of protein y (output signal)
function y = driver_det(x, m0, y0, dt, t_end, plot_on)

% parameters
k1 = 0.00012;       % association of x to operator OR3
k2 = 0.4791;        % dissocaiation of x from operator OR3
k3 = 0.0715;        % transcription of mRNA of protein y
k4 = 0.043;         % translation of protein y
k5 = 0.0039;        % degradation of protein y mRNA
k6 = 0.0007;        % degradation of protein y

alpha       = k3;
alpha0      = 0;
beta        = k4;
gamma       = 2;
Kd          = (k2/k1)^(1/gamma);
delta_m     = k5;
delta_y     = k6;

N = t_end/dt;
m = zeros(1,N+1);
y = zeros(1,N+1);

m(1) = m0;
y(1) = y0;

i = 2;

for t=dt:dt:t_end

    dm = alpha * x(i-1)^(gamma)/(Kd^gamma + x(i-1)^gamma) + alpha0 - delta_m*m(i-1);
    dy = beta * m(i-1) - delta_y*y(i-1);
    
    m(i) = m(i-1) + dm * dt;
    y(i) = y(i-1) + dy * dt;    
    
    i = i + 1;
end;

if (plot_on == 1)
	figure;
	clf;

	hold on;

	xlabel('Protein x [nM]');
	xlabel('Time [s]');
	ylabel('Protein y [nM]');

	t = [0:dt:t_end];
	plot(t,y);

	hold off;
end;

end

