% stochastic model of the biological driver (inputs are forced by external values)
%
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions
% observ: observed chemical species
% plot_on: turns the plotting on
% inputs: vector of indeces of chemical species that are forced to input_vals
% input_vals: matrix with the values that are forced to input chemical species
%
% Outputs:
% X_means: time course of observed chemical species
function [X_means] = driver_ssa_inputs(t_end,t_step,X_init,observ, plot_on, inputs, input_vals)

runs = 1;

disp('SSA - biological driver');

% -> define the rate constant vector c
A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

% parameters
k1 = 0.00012;       % association of x and operator OR3
k2 = 0.4791;        % dissociation of x from operator OR3
k3 = 0.0715;        % transcription of protein y mRNA
k4 = 0.043;         % translation of protein y
k5 = 0.0039;        % degradation of mRNA
k6 = 0.0007;        % degradation of protein y

c1 = k1*2*10^18/(AV^2); 
c2 = k2;            
c3 = k3;            
c4 = k4;            
c5 = k5;            
c6 = k6;            

c = [c1, c2, c3, c4, c5, c6];


% -> define the reactions (i.e. the reactants and the products matrix)
% X = [x, DNK, x2DNK, mRNK, y]

r1 = [2, 1, 0, 0, 0]; p1 = [0, 0, 1, 0, 0];    % x + x + DNK -> x2_DNK
r2 = [0, 0, 1, 0, 0]; p2 = [2, 1, 0, 0, 0];    % x2_DNK -> x + x  + DNK
r3 = [0, 0, 1, 0, 0]; p3 = [0, 0, 1, 1, 0];    % x2DNK -> mRNK + x2DNK
r4 = [0, 0, 0, 1, 0]; p4 = [0, 0, 0, 1, 1];    % mRNK -> mRNK + y
r5 = [0, 0, 0, 1, 0]; p5 = [0, 0, 0, 0, 0];    % mRNK -> 0
r6 = [0, 0, 0, 0, 1]; p6 = [0, 0, 0, 0, 0];    % y -> 0

reactants = [r1;r2;r3;r4;r5;r6];
products = [p1;p2;p3;p4;p5;p6];


% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps
t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

for k=1:runs
    [X] = ssa_input(c,reactants,products,X_init,t_end,t_step,inputs,input_vals);
    
    for l=1:length(observ)
        X_total(k,l,:)=X(observ(l),:);
    end;
end;

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)

    for l=1:length(observ)     
        figure, plot(t, X_means(l,:));
        xlabel('Time [s]');
        ylabel('Number of molecules of protein y');
        box off;
    end;
end;