% stochastic model of the NOR gate
%
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions
% observ: observed chemical species
% plot_on: turns the plotting on
%
% Outputs:
% X_mean: time course of observed chemical species
%
% sample calls
%X_init = [0,0,1,0,0,25,1200]'; %a
%X_init = [0,600,1,0,0,25,1200]'; %b
%X_init = [600,0,1,0,0,25,1200]'; %c
%X_init = [600,600,1,0,0,25,1200]'; %d
% nor_ssa(10000,10,X_init,[7],1);
%
function [X_mean] = nor_ssa(t_end,t_step,X_init,observ,plot_on)

% X = [x, y, DNK, x2DNK, y2DNK, mRNK, z]

runs = 1;

disp('SSA - NOR gate');

% -> define the rate constant vector c
A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

% parameters
k1 = 0.00012;       % association of x and operator OR3
k2 = 0.4791;        % dissociation of x from operator OR3
k3 = 0.00012;       % association of y and operator OR3
k4 = 0.4791;        % dissociation of y from operator OR3
k5 = 0.0715;        % transcription of protein z mRNA
k6 = 0.043;         % translation of protein z
k7 = 0.0039;        % degradation of mRNA
k8 = 0.0007;        % degradation of protein z

c1 = k1*2*10^18/(AV^2);  
c2 = k2;                 
c3 = k3*2*10^18/(AV^2);  
c4 = k4;                 
c5 = k5;                 
c6 = k6;                 
c7 = k7;                 
c8 = k8;                 

c = [c1, c2, c3, c4, c5, c6, c7, c8];

% -> define the reactions (i.e. the reactants and the products matrix)
% X = [x, y, DNK, x2DNK, y2DNK, mRNK, z]

r1 = [2, 0, 1, 0, 0, 0, 0]; p1 = [0, 0, 0, 1, 0, 0, 0]; % x + x + DNK -> x2_DNK
r2 = [0, 0, 0, 1, 0, 0, 0]; p2 = [2, 0, 1, 0, 0, 0, 0]; % x2_DNK -> x + x  + DNK
r3 = [0, 2, 1, 0, 0, 0, 0]; p3 = [0, 0, 0, 0, 1, 0, 0]; % y + y + DNK -> y2_DNK
r4 = [0, 0, 0, 0, 1, 0, 0]; p4 = [0, 2, 1, 0, 0, 0, 0]; % y2_DNK -> y + y  + DNK
r5 = [0, 0, 1, 0, 0, 0, 0]; p5 = [0, 0, 1, 0, 0, 1, 0]; % DNK -> mRNK + DNK
r6 = [0, 0, 0, 0, 0, 1, 0]; p6 = [0, 0, 0, 0, 0, 1, 1]; % mRNK -> mRNK + z
r7 = [0, 0, 0, 0, 0, 1, 0]; p7 = [0, 0, 0, 0, 0, 0, 0]; % mRNK -> 0
r8 = [0, 0, 0, 0, 0, 0, 1]; p8 = [0, 0, 0, 0, 0, 0, 0]; % z -> 0

reactants = [r1;r2;r3;r4;r5;r6;r7;r8];
products = [p1;p2;p3;p4;p5;p6;p7;p8];


% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps
t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

for k=1:runs
    [X] = ssa(c,reactants,products,X_init,t_end,t_step);
   
    for l=1:length(observ)
        X_total(k,l,:)=X(observ(l),:);
    end;
end;

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)

    for l=1:length(observ)
        figure(l), plot(t, X_means(l,:));
        axis([0,t_end,0,1400]);
        xlabel('Time [s]');
        ylabel('Number of molecules of protein z');
        box off;
    end;
end