% searches for the location of the bifurcation point in the extended repressilator model
% plot the bifurcations diagram (stable point: o, unstable points: x)
% Inputs:
% bifur_param: options are case 'alpha', 'alpha0', 'beta', 'gamma', 'Kd', 'delta_m', 'delta'; default = alpha
% start,stop: interval for the searcg of a bifurcation point and initial step
% step: initial step size
% error: error tolerance
%
% Outputs:
% loc: location of the bifurcation point; NaN if not found
%
% Sample call
% [loc] = repressilator_extended_bifurcation('delta_m', 0.0007,1,0.001, 10^(-6), 1)
function [loc] = repressilator_extended_bifurcation(bifur_param, start,stop,step, error, plot_on)

if (plot_on == 1)
	figure(1)  % protein x
	clf;
	hold on;
	xlabel('param');
	ylabel('Protein x [nM]');
end;
	
	
% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

% cell volume
V = 2*10^(-15);

%
alphax_       = k_transcription;
alpha0x_      = 0;
betax_        = k_translation;
gammax_       = 2;
Kdx_         = (k_dissociation_OR2/k_association_OR2)^(1/gammax_); 

alphay_       = k_transcription;
alpha0y_      = 0;
betay_        = k_translation;
gammay_       = 2;
Kdy_         = (k_dissociation_OR2/k_association_OR2)^(1/gammay_); 

alphaz_       = k_transcription;
alpha0z_      = 0;
betaz_        = k_translation;
gammaz_       = 2;
Kdz_         = (k_dissociation_OR2/k_association_OR2)^(1/gammaz_); 

alphaa_       = k_transcription;
alpha0a_      = 0;
betaa_        = k_translation;
gammaa_       = 2;
Kda_         = (k_dissociation_OR2/k_association_OR2)^(1/gammaa_); 

alphab_       = k_transcription;
alpha0b_      = 0;
betab_        = k_translation;
gammab_       = 2;
Kdb_         = (k_dissociation_OR2/k_association_OR2)^(1/gammab_); 


delta_mx_     = k_degrad_mRNA;
delta_my_     = k_degrad_mRNA;
delta_mz_     = k_degrad_mRNA;
delta_ma_     = k_degrad_mRNA;
delta_mb_     = k_degrad_mRNA;

delta_x_     = k_degrad_mono;
delta_y_     = k_degrad_mono;
delta_z_     = k_degrad_mono;
delta_a_     = k_degrad_mono;
delta_b_     = k_degrad_mono;

% search for a bifurcation point
while (abs(step) > error)
    

    n_stable_old = -1;      
    old_step = step;   

    
     for (param = start:step:stop)
        
		switch bifur_param 
			case 'alpha'
				alphax_ = param;
			case 'alpha0'
				alpha0x_ = param;
			case 'beta'
				betax_ = param;
			case 'gamma'
				gammax_ = param;
			case 'Kd'
				 Kdx_ = param;
			case 'delta_m'
				delta_mx_ = param;
			case 'delta'
				delta_x_ = param;
			otherwise
				warning('Invalid parameter name! Setting to alpha!');
				alphax_ = param;
        end;
        
    % differential equations
    syms x y z a b mx my mz ma mb
        syms alphax alpha0x betax gammax Kdx delta_mx delta_x
        syms alphay alpha0y betay gammay Kdy delta_my delta_y
        syms alphaz alpha0z betaz gammaz Kdz delta_mz delta_z
		syms alphaa alpha0a betaa gammaa Kda delta_ma delta_a
		syms alphab alpha0b betab gammab Kdb delta_mb delta_b
        
        dm_x = 'alphax/(1 + (b/Kdx)^gammax) + alpha0x - mx * delta_mx';
        dm_y = 'alphay/(1 + (x/Kdy)^gammay) + alpha0y - my * delta_my';
        dm_z = 'alphaz/(1 + (y/Kdz)^gammaz) + alpha0z - mz * delta_mz';
		dm_a = 'alphaa/(1 + (z/Kda)^gammaa) + alpha0a - ma * delta_ma';
		dm_b = 'alphab/(1 + (a/Kdb)^gammab) + alpha0b - mb * delta_mb';
    
        dx = 'betax * mx - delta_x * x'; 
        dy = 'betay * my - delta_y * y'; 
        dz = 'betaz * mz - delta_z * z';
		da = 'betaa * ma - delta_a * a';
		db = 'betab * mb - delta_b * b';
        
        dm_x = subs(dm_x,[alphax, alpha0x, gammax, Kdx, delta_mx], [alphax_, alpha0x_, gammax_, Kdx_, delta_mx_]);
        dm_y = subs(dm_y,[alphay, alpha0y, gammay, Kdy, delta_my], [alphay_, alpha0y_, gammay_, Kdy_, delta_my_]);
        dm_z = subs(dm_z,[alphaz, alpha0z, gammaz, Kdz, delta_mz], [alphaz_, alpha0z_, gammaz_, Kdz_, delta_mz_]);
		dm_a = subs(dm_a,[alphaa, alpha0a, gammaa, Kda, delta_ma], [alphaa_, alpha0a_, gammaa_, Kda_, delta_ma_]);
		dm_b = subs(dm_b,[alphab, alpha0b, gammab, Kdb, delta_mb], [alphab_, alpha0b_, gammab_, Kdb_, delta_mb_]);

        dx = subs(dx,[betax, delta_x], [betax_, delta_x_]);
        dy = subs(dy,[betay, delta_y], [betay_, delta_y_]);
        dz = subs(dz,[betaz, delta_z], [betaz_, delta_z_]);                  
		da = subs(da,[betaa, delta_a], [betaa_, delta_a_]);
		db = subs(db,[betab, delta_b], [betab_, delta_b_]);

        % determine steady states (d.../dt = 0)
        S = solve(dm_x, dm_y, dm_z, dm_a, dm_b, dx, dy, dz, da, db);

        n_stable = 0;
        solution = 0;
        
        if (length(S) > 0)
             solution = 1;
             
					 
             j = 1;
             for i=1:length(S.x)
                 if ((double(S.x(i)) >= 0) && (double(S.y(i)) >= 0) && (double(S.z(i)) >= 0) && (double(S.a(i)) >= 0) && (double(S.b(i)) >= 0) && (imag(double(S.x(i))) == 0) && (imag(double(S.y(i))) == 0) && (imag(double(S.z(i))) == 0) && (imag(double(S.a(i))) == 0) && (imag(double(S.b(i))) == 0))
                     if ((double(S.mx(i)) >= 0) && (double(S.my(i)) >= 0) && (double(S.mz(i)) >= 0)  && (double(S.ma(i)) >= 0)  && (double(S.mb(i)) >= 0) && (imag(double(S.mx(i))) == 0) && (imag(double(S.my(i))) == 0) && (imag(double(S.mz(i))) == 0) && (imag(double(S.ma(i))) == 0) && (imag(double(S.mb(i))) == 0))
                         sol_x(j) = S.x(i);
                         sol_y(j) = S.y(i);
                         sol_z(j) = S.z(i);
						 sol_a(j) = S.a(i);
						 sol_b(j) = S.b(i);
                         sol_mx(j) = S.mx(i);
                         sol_my(j) = S.my(i);                   
                         sol_mz(j) = S.mz(i);                                           
						 sol_ma(j) = S.ma(i);                                           
						 sol_mb(j) = S.mb(i);                                           
                         j=j+1;
                     end;
                 end;
             end;
 
             j = j-1;
 
             sol_x = double(sol_x);
             sol_y = double(sol_y);
             sol_z = double(sol_z);
			 sol_a = double(sol_a);
			 sol_b = double(sol_b);
             sol_mx = double(sol_mx);
             sol_my = double(sol_my);        
             sol_mz = double(sol_mz);        
			 sol_ma = double(sol_ma);        
			 sol_mb = double(sol_mb);         


            % Jacobian
            % J = [diff(dx_dt,x), diff(dx_dt,y); diff(dy_dt,x), diff(dy_dt,y)];
            J = [diff(dx,x), diff(dx,y), diff(dx,z), diff(dx,mx), diff(dx,my), diff(dx,mz), diff(dx,ma), diff(dx,mb);  
                  diff(dy,x), diff(dy,y), diff(dy,z), diff(dy,mx), diff(dy,my), diff(dy,mz), diff(dy,ma), diff(dy,mb); 
                  diff(dz,x), diff(dz,y), diff(dz,z), diff(dz,mx), diff(dz,my), diff(dz,mz), diff(dz,ma), diff(dz,mb);
				  diff(da,x), diff(da,y), diff(da,z), diff(da,mx), diff(da,my), diff(da,mz), diff(da,ma), diff(da,mb);
				  diff(db,x), diff(db,y), diff(db,z), diff(db,mx), diff(db,my), diff(db,mz), diff(db,ma), diff(db,mb);
                  diff(dm_x,x), diff(dm_x,y), diff(dm_x,z), diff(dm_x,mx), diff(dm_x,my), diff(dm_x,mz), diff(dm_x,ma), diff(dm_x,mb);  
                  diff(dm_y,x), diff(dm_y,y), diff(dm_y,z), diff(dm_y,mx), diff(dm_y,my), diff(dm_y,mz), diff(dm_y,ma), diff(dm_y,mb); 
                  diff(dm_z,x), diff(dm_z,y), diff(dm_z,z), diff(dm_z,mx), diff(dm_z,my), diff(dm_z,mz), diff(dm_z,ma), diff(dm_z,mb);
				  diff(dm_a,x), diff(dm_a,y), diff(dm_a,z), diff(dm_a,mx), diff(dm_a,my), diff(dm_a,mz), diff(dm_a,ma), diff(dm_a,mb);
				  diff(dm_b,x), diff(dm_b,y), diff(dm_b,z), diff(dm_b,mx), diff(dm_b,my), diff(dm_b,mz), diff(dm_b,ma), diff(dm_b,mb)];      
				  
            % verify the stability for each steady state

            eJ = zeros(j,10); ; 

            
            for i=1:j
                 eJ(i,:) = eig(subs(J,[x,y,z,a,b,mx,my,mz,ma,mb],[sol_x(i),sol_y(i),sol_z(i),sol_a(i),sol_b(i),sol_mx(i),sol_my(i),sol_mz(i),sol_ma(i),sol_mb(i)]));
 
                 ex = double(eJ(i,1));
                 ey = double(eJ(i,2));
                 ez = double(eJ(i,3));
				 ea = double(eJ(i,4));
				 eb = double(eJ(i,5));
                 emx = double(eJ(i,6));
                 emy = double(eJ(i,7));
                 emz = double(eJ(i,8));
				 ema = double(eJ(i,9));
				 emb = double(eJ(i,10));
                %fprintf('Eigen values for [x,y,mx,my] = [%d, %d, %d, %d] are (%d, %d, %d, %d)\n',double(sol_x(i)),double(sol_y(i)),double(sol_mx(i)),double(sol_my(i)),ex,ey,emx,emy);

                if (ex < 0) && (ey < 0) && (ez < 0) && (ea < 0) && (eb < 0) && (emx < 0) && (emy < 0) && (emz < 0) && (ema < 0) && (emb < 0)
					if (plot_on == 1)
						figure(1)
                        plot(param,double(sol_x(i)),'o');                        
					end;
                    n_stable = n_stable + 1;
                  else
					if (plot_on == 1)
						figure(1)
                        plot(param,double(sol_x(i)),'x');                      
					end;
                  end;        

				end; 
                                    
        end;
        
        if (n_stable_old == -1)
             n_stable_old = n_stable;
        end;
         
         
        if (n_stable ~= n_stable_old) && (solution > 0)
            start = max(eps,param - step);
            stop = param;
             
            step = step / 2;
            break
        end;      
                 
                 
         
     end;
     
     if (old_step == step)
         step = 0;       
         param = -1;
     end; 
                
end;

if (param > 0)
    loc = param;
else
    fprintf('Bifurcation point not found in the given interval!\n');
	loc = NaN;
end;

if (plot_on == 1)
	figure(1)
	hold off;	
end;
end