% deterministic model of the repressilator with 5 elements
%
% Inputs:
% init: initial state [m_x, x, m_y, y, m_z, z, m_a, a, m_b, b]
% dt: time step size
% t_end: simulation time
% plot_on: turns the plotting on
% 
% Outputs:
% x: time evolution of protein x (output signal)
function x = repressilator_extended_det(init, dt, t_end, plot_on)


% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

% cell volume
V = 2*10^(-15);

alpha       = k_transcription;
alpha0      = 0;
beta        = k_translation;
gamma       = 2;
Kd          = (k_dissociation_OR2/k_association_OR2)^(1/gamma);

delta_m     = k_degrad_mRNA;
delta_x     = k_degrad_mono;
delta_y     = k_degrad_mono;
delta_z     = k_degrad_mono;
delta_a     = k_degrad_mono;
delta_b     = k_degrad_mono;



N = t_end/dt;

m_x = zeros(1,N+1);
m_y = zeros(1,N+1);
m_z = zeros(1,N+1);
m_a = zeros(1,N+1);
m_b = zeros(1,N+1);
x = zeros(1,N+1);
y = zeros(1,N+1);
z = zeros(1,N+1);
a = zeros(1,N+1);
b = zeros(1,N+1);

% initialisation
m_x(1)  = init(1); %0;
x(1)    = init(2); %200;
m_y(1)  = init(3); %0;
y(1)    = init(4); %0;
m_z(1)  = init(5); %0;
z(1)    = init(6); %0;
m_a(1)  = init(7); %0;
a(1)    = init(8); %0;
m_b(1)  = init(9); %0;
b(1)    = init(10); %0;

i = 2;

for t=dt:dt:t_end
  
    curr_x  = x(i-1);
    curr_y  = y(i-1);
    curr_z  = z(i-1);
    curr_a  = a(i-1);
    curr_b  = b(i-1);
    curr_mx = m_x(i-1);
    curr_my = m_y(i-1);
    curr_mz = m_z(i-1);
    curr_ma = m_a(i-1);
    curr_mb = m_b(i-1);
    
    dm_x = alpha/(1 + (curr_b/Kd)^gamma) + alpha0 - curr_mx * delta_m;   
    dm_y = alpha/(1 + (curr_x/Kd)^gamma) + alpha0 - curr_my * delta_m;
    dm_z = alpha/(1 + (curr_y/Kd)^gamma) + alpha0 - curr_mz * delta_m;
    dm_a = alpha/(1 + (curr_z/Kd)^gamma) + alpha0 - curr_ma * delta_m;
    dm_b = alpha/(1 + (curr_a/Kd)^gamma) + alpha0 - curr_mb * delta_m;
    
    dx = beta * curr_mx - delta_x * curr_x; 
    dy = beta * curr_my - delta_y * curr_y; 
    dz = beta * curr_mz - delta_z * curr_z;
    da = beta * curr_ma - delta_a * curr_a;
    db = beta * curr_mb - delta_b * curr_b;

    m_x(i) = curr_mx + dm_x * dt;
    m_y(i) = curr_my + dm_y * dt;
    m_z(i) = curr_mz + dm_z * dt;
    m_a(i) = curr_ma + dm_a * dt;
    m_b(i) = curr_mb + dm_b * dt;
    
    x(i)    = curr_x + dx * dt;
    y(i)    = curr_y + dy * dt;
    z(i)    = curr_z + dz * dt;
    a(i)    = curr_a + da * dt;
    b(i)    = curr_b + db * dt;
      
    i = i + 1;
end;

if (plot_on==1)
	figure;
	clf;

	hold on;

	t = [0:dt:t_end];
	plot(t,x,'b');
	plot(t,y,'r');
	plot(t,z,'g');
	plot(t,a,'c');
	plot(t,b,'m');


	xlabel('Time [s]');
	ylabel('Protein [nM]');

	hold off;
end;



end