% searches for the location of the bifurcation point in the toggle switch model
% plot the bifurcations diagram (stable point: o, unstable points: x)
% Inputs:
% bifur_param: options are case 'alpha', 'alpha0', 'beta', 'gamma', 'Kd1', 'Kd2', 'delta_m', 'delta'; default = alpha
% start,stop: interval for the searcg of a bifurcation point and initial step
% step: initial step size
% error: error tolerance
%
% Outputs:
% loc: location of the bifurcation point; NaN if not found
%
% Sample call
% [loc] = toggle_bifurcation('delta_m', 0.0039,0,-0.0001, 10^(-6), 1)
function [loc] = toggle_bifurcation(bifur_param, start,stop,step, error, plot_on)

if (plot_on==1)
	figure(1)  % protein x
	clf;
	hold on;
	xlabel('param');
	ylabel('Protein x [nM]');

	figure(2)  % protein y
	clf;
	hold on;
	xlabel('param');
	ylabel('Protein y [nM]');
end;
	
	
% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNK   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

% cell volume
V = 2*10^(-15);

%
alphax_       = k_transcription;
alpha0x_      = 0;
betax_        = k_translation;
gammax_       = 2;
Kd1x_         = (k_dissociation_OR2/k_association_OR2)^(1/gammax_);
Kd2x_         = (k_dissociation_OR2/k_association_OR3)^(1/gammax_);

alphay_       = k_transcription;
alpha0y_      = 0;
betay_        = k_translation;
gammay_       = 2;
Kd1y_         = (k_dissociation_OR2/k_association_OR2)^(1/gammay_);
Kd2y_         = (k_dissociation_OR2/k_association_OR3)^(1/gammay_);

delta_mx_     = k_degrad_mRNK;
delta_my_     = k_degrad_mRNK;

delta_x_     = k_degrad_mono;
delta_y_     = k_degrad_mono;

% looking for a bifurcation point in the absence of external inputs
rep_i_       = 0;	
rep_j_       = 0;

% iskanje tocke bifurkacije
while (abs(step) > error)
    

    n_stable_old = -1;      
    old_step = step;   

    
     for (param = start:step:stop)
        
		switch bifur_param 
			case 'alpha'
				alphax_ = param;
			case 'alpha0'
				alpha0x_ = param;
			case 'beta'
				betax_ = param;
			case 'gamma'
				gammax_ = param;
			case 'Kd1'
				 Kd1x_ = param;
			case 'Kd2'
				Kd2x_ = param;
			case 'delta_m'
				delta_mx_ = param;
			case 'delta'
				delta_x_ = param;
			otherwise
				warning('Invalid parameter name! Setting to alpha!');
				alphax_ = param;
        end;
        
    % differential equations
    syms x y mx my rep_i rep_j
    syms alphax alpha0x betax gammax Kd1x Kd2x delta_mx 
    syms alphay alpha0y betay gammay Kd1y Kd2y delta_my 
    syms delta_x delta_y 
   
    dm_x = 'alphax/(1 + (y/Kd2x)^gammax + (rep_i/Kd1x)^gammax) + alpha0x - mx * delta_mx';   
    dm_y = 'alphay/(1 + (x/Kd2y)^gammay + (rep_j/Kd1y)^gammay) + alpha0y - my * delta_my';    
    
    dx = 'betax * mx - delta_x * x'; 
    dy = 'betay * my - delta_y * y'; 
         
    dm_x = subs(dm_x,[alphax, alpha0x, gammax, Kd1x, Kd2x, delta_mx, rep_i], [alphax_, alpha0x_, gammax_, Kd1x_, Kd2x_, delta_mx_, rep_i_]);
    dm_y = subs(dm_y,[alphay, alpha0y, gammay, Kd1y, Kd2y, delta_my, rep_j], [alphay_, alpha0y_, gammay_, Kd1y_, Kd2y_, delta_my_, rep_j_]);
        
    dx = subs(dx,[betax, delta_x], [betax_, delta_x_]);
    dy = subs(dy,[betay, delta_y], [betay_, delta_y_]);

        % determine steady states (d.../dt = 0)
        S = solve(dm_x, dm_y, dx, dy);

        n_stable = 0;
        solution = 0;
        
        if (length(S) > 0)
            solution = 1;
            
            j = 1;
            for i=1:length(S.x)
                if ((double(S.x(i)) >= 0) && (double(S.y(i)) >= 0) && (imag(double(S.x(i))) == 0) && (imag(double(S.y(i))) == 0))
                    if ((double(S.mx(i)) >= 0) && (double(S.my(i)) >= 0) && (imag(double(S.mx(i))) == 0) && (imag(double(S.my(i))) == 0))
                        sol_x(j) = S.x(i);
                        sol_y(j) = S.y(i);
                        sol_mx(j) = S.mx(i);
                        sol_my(j) = S.my(i);                   
                        j=j+1;
                    end;
                end;
            end;

            j = j-1;

            sol_x = double(sol_x);
            sol_y = double(sol_y);
            sol_mx = double(sol_mx);
            sol_my = double(sol_my);        


            % Jacobian
            % J = [diff(dx_dt,x), diff(dx_dt,y); diff(dy_dt,x), diff(dy_dt,y)];
            J = [diff(dx,x), diff(dx,y), diff(dx,mx), diff(dx,my);  
                 diff(dy,x), diff(dy,y), diff(dy,mx), diff(dy,my);  
                 diff(dm_x,x), diff(dm_x,y), diff(dm_x,mx), diff(dm_x,my);  
                 diff(dm_y,x), diff(dm_y,y), diff(dm_y,mx), diff(dm_y,my)];

             % verify the stability for each steady state

            eJ = zeros(j,4); 

            
            for i=1:j
                eJ(i,:) = eig(subs(J,[x,y,mx,my],[sol_x(i),sol_y(i),sol_mx(i),sol_my(i)]));

                ex = double(eJ(i,1));
                ey = double(eJ(i,2));
                emx = double(eJ(i,3));
                emy = double(eJ(i,4));
                %fprintf('Eigen values for [x,y,mx,my] = [%d, %d, %d, %d] are (%d, %d, %d, %d)\n',double(sol_x(i)),double(sol_y(i)),double(sol_mx(i)),double(sol_my(i)),ex,ey,emx,emy);

                if (ex < 0) && (ey < 0) && (emx < 0) && (emy < 0)
					if (plot_on == 1)
						figure(1)
                        plot(param,double(sol_x(i)),'o');
                        figure(2)
                        plot(param,double(sol_y(i)),'o');
					end;
                    n_stable = n_stable + 1;
                  else
					if (plot_on == 1)
						figure(1)
                        plot(param,double(sol_x(i)),'x');
                        figure(2)
                        plot(param,double(sol_y(i)),'x');
					end;
                  end;        

				end; 
                                    
        end;
        
        if (n_stable_old == -1)
            n_stable_old = n_stable;
        end;
        
        
        if (n_stable ~= n_stable_old) && (solution > 0)
            start = max(eps,param - step);
            stop = param;
            
            step = step / 2;
            break
        end;      
                
                
        
    end;
    
    if (old_step == step)
        step = 0;       
        param = -1;
    end;

    
end;

if (param > 0)
    loc = param;
else
    fprintf('Bifurcation point not found in the given interval!\n');
	loc = NaN;
end;

if (plot_on == 1)
	figure(1)
	hold off;
	figure(2)
	hold off;
end;
end