% deterministic model of the toggle switch
%
% Inputs:
% rep_i, rep_j: input signals
% init: initial state [m_x, x, m_y, y]
% dt: time step size
% t_end: simulation time
% plot_on: turns the plotting on
% 
% Outputs:
% x,y: time evolution of proteins x and y (output signals)
%
function [x,y] = toggle_det(rep_i, rep_j, init, dt, t_end, plot_on);


% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_degrad_fast   = 1;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

% cell volume
V = 2*10^(-15);

alpha       = k_transcription;
alpha0      = 0;
beta        = k_translation;
gamma       = 2;
Kd1         = (k_dissociation_OR2/k_association_OR2)^(1/gamma);
Kd2         = (k_dissociation_OR2/k_association_OR3)^(1/gamma);

delta_m     = k_degrad_mRNA;
delta_x     = k_degrad_mono;
delta_y     = k_degrad_mono;

N = t_end/dt;

m_x = zeros(1,N+1);
x = zeros(1,N+1);
m_y = zeros(1,N+1);
y = zeros(1,N+1);

%%%
% initialisation

% (a)
m_x(1) = init(1); %20;
x(1) = init(2); %1100;
m_y(1) = init(3); %0;
y(1) = init(4); %0;

% (b)
%m_x(1) = 0;
%x(1) = 0;
%m_y(1) = 0;
%y(1) = 200;

i = 2;

for t=dt:dt:t_end
    curr_x  = x(i-1);
    curr_y  = y(i-1);
    curr_mx = m_x(i-1);
    curr_my = m_y(i-1);
    
    
    dm_x = alpha/(1 + (rep_i(i-1)/Kd2)^gamma + (curr_y/Kd1)^gamma) + alpha0 - curr_mx * delta_m;   
    dm_y = alpha/(1 + (rep_j(i-1)/Kd2)^gamma + (curr_x/Kd1)^gamma) + alpha0 - curr_my * delta_m;   
   
    dx = beta * curr_mx - delta_x * curr_x; 
    dy = beta * curr_my - delta_y * curr_y; 

    m_x(i) = curr_mx + dm_x * dt;
    m_y(i) = curr_my + dm_y * dt;
    
    x(i)    = curr_x + dx * dt;
    y(i)    = curr_y + dy * dt;
      
    i = i + 1;
end;

if (plot_on == 1)

	figure;
	clf;

	hold on;

	t = [0:dt:t_end];
	plot(t,x,'b');
	plot(t,y,'r');

	xlabel('Time [s]');
	ylabel('Proteins x and y [nM]');

	hold off;
end;



