% stochastic model of the toggle switch 
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions
% observ: observed chemical species
% plot_on: turns the plotting on
%
% Outputs:
% X_means: time course of observed chemical species
%
% 	example of call:
% 	X_init = [1200, 2, 0, 0, 1, 1, 0, 0, 0, 0, 20, 0,]';
%   toggle_ssa(10000,1,X_init,[1,2],1)
%
function [X_mean] = toggle_ssa(t_end,t_step,X_init,observ, plot_on)

runs = 1;

% X = [x, y, rep_i, rep_j, DNA_x, DNA_y, x2_DNA_y, y2_DNA_x, rep_i_DNA_x, rep_j_DNA_y, mRNA_x, mRNA_y]




disp('SSA - toggle switch');

% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_associtation_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_associtation_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;


[reactants, products, c] = setParams(k_translation, k_degrad_mono, k_transcription, k_degrad_mRNA, k_associtation_OR2, k_dissociation_OR2, k_associtation_OR3, k_dissociation_OR3, k_dimer, k_mono);

% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps


t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

X_init_tmp = X_init;



for k=1:runs
 
    X_init = X_init_tmp;
	
	
    [X] = ssa(c,reactants,products,X_init,t_end,t_step);
 
    for l=1:length(observ)
        X_total(k,l,:)=X(observ(l),:);
    end;
    
end;

    

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)
        	
	prot_x = X_means(:,1);
    prot_y = X_means(:,2);
  
  
    
    figure(1)
    plot(t,prot_x);
    axis([0,t_end,0,1500]);
    hold on;
    plot(t,prot_y,'r');
    xlabel('Time [s]');
    ylabel('Number of molecules of proteins x and y');
    box off;
    hold off;
    
    
    
end
end

function [reactants, products, c] = setParams(k_translation, k_degrad_mono, k_transcription, k_degrad_mRNA, k_associtation_OR2, k_dissociation_OR2, k_associtation_OR3, k_dissociation_OR3, k_dimer, k_mono)

% -> define the rate constant vector c
A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

scale_bi = 10^9/AV;
scale_tri = 2*10^18/(AV^2);

k_degrad_mono_x = k_degrad_mono;
k_degrad_mono_y = k_degrad_mono;


% X = [x, y, rep_i, rep_j, DNA_x, DNA_y, x2_DNA_y, y2_DNA_x, rep_i_DNA_x,
% rep_j_DNA_y, mRNA_x, mRNA_y]


x = 1;
y = 2;
rep_i = 3;
rep_j = 4;
DNA_x = 5;
DNA_y = 6;
x2_DNA_y = 7;
y2_DNA_x = 8;
rep_i_DNA_x = 9;
rep_j_DNA_y = 10;
mRNA_x = 11;
mRNA_y = 12;

r1  = zeros(1,12);
r2  = zeros(1,12);
r3  = zeros(1,12);
r4  = zeros(1,12);
r5  = zeros(1,12);
r6  = zeros(1,12);
r7  = zeros(1,12);
r8  = zeros(1,12);
r9  = zeros(1,12);
r10 = zeros(1,12);
r11 = zeros(1,12);
r12 = zeros(1,12);
r13 = zeros(1,12);
r14 = zeros(1,12);
r15 = zeros(1,12);
r16 = zeros(1,12);
r17 = zeros(1,12);
r18 = zeros(1,12);

p1  = zeros(1,12);
p2  = zeros(1,12);
p3  = zeros(1,12);
p4  = zeros(1,12);
p5  = zeros(1,12);
p6  = zeros(1,12);
p7  = zeros(1,12);
p8  = zeros(1,12);
p9  = zeros(1,12);
p10 = zeros(1,12);
p11 = zeros(1,12);
p12 = zeros(1,12);
p13 = zeros(1,12);
p14 = zeros(1,12);
p15 = zeros(1,12);
p16 = zeros(1,12);
p17 = zeros(1,12);
p18 = zeros(1,12);


% [x] + [x] + [DNA_y]    -> [x2_DNA_y]

r1(x) = 2;
r1(DNA_y) = 1;

p1(x2_DNA_y) = 1;

k1 = k_associtation_OR2;
c1 = scale_tri * k1;

% [y] + [y] + [DNA_x]    -> [y2_DNA_x]
r2(y) = 2;
r2(DNA_x) = 1;

p2(y2_DNA_x) = 1;

k2 = k_associtation_OR2;
c2 = scale_tri * k2;

% [x2_DNA_y]        -> [x] + [x] + [DNA_y]
r3(x2_DNA_y) = 1;

p3(x) = 2;
p3(DNA_y) = 1;

k3 = k_dissociation_OR2;
c3 = k3;

% [y2_DNA_x]        -> [y] + [y] + [DNA_x]
r4(y2_DNA_x) = 1;

p4(y) = 2;
p4(DNA_x) = 1;

k4 = k_dissociation_OR2;
c4 = k4;

% [rep_i] + [rep_i] + [DNA_x]    -> [rep_i_DNA_x]
r5(rep_i) = 2;
r5(DNA_x) = 1;

p5(rep_i_DNA_x) = 1;

k5 = k_associtation_OR3;
c5 = scale_tri * k5;

% [rep_j] + [rep_j] + [DNA_y]    -> [rep_j_DNA_y]
r6(rep_j) = 2;
r6(DNA_y) = 1;

p6(rep_j_DNA_y) = 1;

k6 = k_associtation_OR3;
c6 = scale_tri * k6;

% [rep_i_DNA_x]        -> [rep_i] + [rep_i] + [DNA_x]
r7(rep_i_DNA_x) = 1;

p7(rep_i) = 2;
p7(DNA_x) = 1;

k7 = k_dissociation_OR2;
c7 = k7;

% [rep_j_DNA_y]        -> [rep_j] + [rep_j] + [DNA_y]
r8(rep_j_DNA_y) = 1;

p8(rep_j) = 2;
p8(DNA_y) = 1;

k8 = k_dissociation_OR2;
c8 = k8;

% [DNA_x]           -> [DNA_x] + [mRNA_x]
r9(DNA_x) = 1;

p9(DNA_x) = 1;
p9(mRNA_x) = 1;

k9 = k_transcription;
c9 = k9;

% [DNA_y]           -> [DNA_y] + [mRNA_y]
r10(DNA_y) = 1;

p10(DNA_y) = 1;
p10(mRNA_y) = 1;

k10 = k_transcription;
c10 = k10;

% [mRNA_x]            -> [mRNA_x] + [X]
r11(mRNA_x) = 1;

p11(mRNA_x) = 1;
p11(x) = 1;

k11 = k_translation;
c11 = k11;

% [mRNA_y]            -> [mRNA_Y] + [Y]
r12(mRNA_y) = 1;

p12(mRNA_y) = 1;
p12(y) = 1;

k12 = k_translation;
c12 = k12;

% X                 -> 0
r13(x) = 1;

k13 = k_degrad_mono_x;
c13 = k13;

% Y                 -> 0
r14(y) = 1;

k14 = k_degrad_mono_y;
c14 = k14;

% X2_DNA_y          -> DNA_y
r15(x2_DNA_y) = 1;

p15(DNA_y) = 1;

k15 = k_degrad_mono_x;
c15 = k15;

% Y2_DNA_x          -> DNA_x
r16(y2_DNA_x) = 1;

p16(DNA_x) = 1;

k16 = k_degrad_mono_y;
c16 = k16;

% mRNA_x            -> 0
r17(mRNA_x) = 1;

k17 = k_degrad_mRNA;
c17 = k17;

% mRNA_y            -> 0
r18(mRNA_y) = 1;

k18 = k_degrad_mRNA;
c18 = k18;


reactants = [r1;r2;r3;r4;r5;r6;r7;r8;r9;r10;r11;r12;r13;r14;r15;r16;r17;r18];
products =  [p1;p2;p3;p4;p5;p6;p7;p8;p9;p10;p11;p12;p13;p14;p15;p16;p17;p18];
c =         [c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18];

end
